@@ -21,6 +21,6 @@ function(twoparameter,nt) {
             probability <- EPPMprob(vlambda) 
                                     } # end of if (b==1)
             } else { 
-                 if (a<0) { probability <- c(1,rep(0,nt))
+                 if (a<=0) { probability <- c(1,rep(0,nt))
                    } else { probability <- c(rep(0,nt),1) }} # end of if a>0 & a>1.e+20
-      return(probability)           }
+      return(probability) }
