@@ -14,8 +14,7 @@ function (x, digits = max(3, getOption("digits") - 3), ...)
         }
         else cat("No coefficients (in mean model)\n\n")
             if (length(x$coefficients$scalef.est)) {
-                cat(paste("Coefficients (model for scale-factor with ", 
-                  x$link, " link):\n", sep = ""))
+                cat(paste("Coefficients (model for scale-factor with log link):\n", sep = ""))
                 print.default(format(x$coefficients$scalef.est, 
                   digits = digits), print.gap = 2, quote = FALSE)
                 cat("\n")
